/*
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2010 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author:                                                              |
   +----------------------------------------------------------------------+
 */

/* $Id: header 297205 2010-03-30 21:09:07Z johannes $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "stdio.h"
#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "pdo/php_pdo.h"
#include "pdo/php_pdo_driver.h"
#include "php_pdo_sqlanywhere.h"
#include "php_pdo_sqlanywhere_int.h"
#include "php_pdo_sqlanywhere_ver.h"

#define _SACAPI_VERSION 2
#include "sacapi.h"
#include "sacapidll.h"

#if defined( WIN32 )

#define DBCAPI_PREFIX   ""
#define DBCAPI_SUFFIX   ".dll"

#if defined( ZTS )
#define DBCAPI_ZTS     ""
#else
#define DBCAPI_ZTS	   ""
#endif

#define DLL_PATH    "PATH"

#else

#define DBCAPI_PREFIX   "lib"
#define DBCAPI_SUFFIX   ".so"

#if defined( ZTS )
#define DBCAPI_ZTS     "_r"
#else
#define DBCAPI_ZTS	   ""
#endif

#define DLL_PATH    "LD_LIBRARY_PATH"

#endif

#define DBCAPI_NAME DBCAPI_PREFIX "dbcapi" DBCAPI_ZTS DBCAPI_SUFFIX
/* api initialization error message */
#define DBCAPI_NOT_FOUND_ERROR "The SQLAnywhere client libraries \
    could not be loaded. Please ensure that " DBCAPI_NAME " can be found in your " DLL_PATH " environment variable."

ZEND_DECLARE_MODULE_GLOBALS(pdo_sqlanywhere);

/* True global resources - no need for thread safety here */
static int le_pdo_sqlanywhere;



/* {{{ pdo_sqlanywhere_functions[]
 *
 * Every user visible function must have an entry in pdo_sqlanywhere_functions[].
 */
const zend_function_entry pdo_sqlanywhere_functions[] = {
    {NULL, NULL, NULL}
};
/* }}} */



/* {{{ pdo_sqlanywhere_module_entry
 *
 * TODO:Might need a ZEND_MODULE_API_NO check. STANDARD_MODULE_HEADER_EX
 *	and the next two lines may be new additions.
 * TODO:Create a proper version number header file. php_sqlanywhere does.
 */
/* establishes dependency relations with PDO */
static zend_module_dep pdo_sqlanywhere_deps[] = {
    ZEND_MOD_REQUIRED("pdo")
    {NULL,NULL,NULL}
};
/* adds pdo_sqlanywhere as a module */
zend_module_entry pdo_sqlanywhere_module_entry = {
    STANDARD_MODULE_HEADER_EX,
    NULL,
    pdo_sqlanywhere_deps,
    "PDO_SQLANYWHERE",
    pdo_sqlanywhere_functions,
    PHP_MINIT(pdo_sqlanywhere),
    PHP_MSHUTDOWN(pdo_sqlanywhere),
    PHP_RINIT(pdo_sqlanywhere),
    PHP_RSHUTDOWN(pdo_sqlanywhere),
    PHP_MINFO(pdo_sqlanywhere),
    PHP_PDO_SQLANYWHERE_VERSION,
    STANDARD_MODULE_PROPERTIES
};
/* }}} */



#ifdef COMPILE_DL_PDO_SQLANYWHERE
ZEND_GET_MODULE(pdo_sqlanywhere)
#endif



    /* {{{ php_pdo_sqlanywhere_init_globals
     */
static void php_pdo_sqlanywhere_init_globals(zend_pdo_sqlanywhere_globals *pdo_sqlanywhere_globals)
{
#if 0
    pdo_sqlanywhere_globals->global_value = 0;
    pdo_sqlanywhere_globals->global_string = NULL;
#endif
    memset( pdo_sqlanywhere_globals, 0, sizeof(*pdo_sqlanywhere_globals) );

}
/* }}} */



/* {{{ PHP_MINIT_FUNCTION
 *
 * Initializes globals and loads dbcapi
 */
PHP_MINIT_FUNCTION(pdo_sqlanywhere)
{

    // initialize globals (besides api)
    ZEND_INIT_MODULE_GLOBALS(pdo_sqlanywhere, php_pdo_sqlanywhere_init_globals, NULL);

    // load dbcapi
    if ( !sqlany_initialize_interface( &(PDO_SQLANYWHERE_G(api)), NULL)) {
	// documents failed load
	php_error_docref(NULL TSRMLS_CC, E_WARNING, DBCAPI_NOT_FOUND_ERROR );
    }

    if (FAILURE == php_pdo_register_driver(&pdo_sqlanywhere_driver)) {
	return FAILURE;
    }

    return SUCCESS;

}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(pdo_sqlanywhere)
{

    // finalize api
    if( !PDO_SQLANYWHERE_G(api).initialized ) {
	return SUCCESS;
    }

    if( PDO_SQLANYWHERE_G(sqlany_init_called) == 1 ) {

	if( PDO_SQLANYWHERE_G(context) ) {
	    PDO_SQLANYWHERE_G(api).sqlany_fini_ex( PDO_SQLANYWHERE_G(context) );
	} else {
	    PDO_SQLANYWHERE_G(api).sqlany_fini();
	}

	PDO_SQLANYWHERE_G(sqlany_init_called) = 0;
    }
    sqlany_finalize_interface( &(PDO_SQLANYWHERE_G(api)) );

    php_pdo_unregister_driver(&pdo_sqlanywhere_driver);
    return SUCCESS;
}
/* }}} */

/* {{{ PHP_RINIT_FUNCTION
 *
 * Initializes dbcapi
 */
PHP_RINIT_FUNCTION(pdo_sqlanywhere)
{
    if( PDO_SQLANYWHERE_G(api).initialized && PDO_SQLANYWHERE_G(sqlany_init_called) == 0 ) {

	if( PDO_SQLANYWHERE_G(api).sqlany_init_ex != NULL ) {
	    PDO_SQLANYWHERE_G(context) = PDO_SQLANYWHERE_G(api).sqlany_init_ex( "PHP", 2 , NULL );

	    if( PDO_SQLANYWHERE_G(context) != NULL ) {
		PDO_SQLANYWHERE_G(sqlany_init_called) = 1;
	    }
	} 
	if( PDO_SQLANYWHERE_G(sqlany_init_called) == 0 && !PDO_SQLANYWHERE_G(api).sqlany_init( "PHP", 1, NULL ) ) {
	    return FAILURE;
	}

	PDO_SQLANYWHERE_G(sqlany_init_called) = 1;
    }

    return SUCCESS;
}
/* }}} */

/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(pdo_sqlanywhere)
{
    return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 *
 */
PHP_MINFO_FUNCTION(pdo_sqlanywhere)
{
    char client_version[32];
    int rc;

    php_info_print_table_start();
    php_info_print_table_header(2, "pdo_sqlanywhere support", "enabled");
    // php_info_print_table_row(...);
    if( PDO_SQLANYWHERE_G(api).initialized ) {
	if( PDO_SQLANYWHERE_G(context) ) {
	    rc = PDO_SQLANYWHERE_G(api).sqlany_client_version_ex( PDO_SQLANYWHERE_G(context), client_version, sizeof(client_version) );
	} else {
	    rc = PDO_SQLANYWHERE_G(api).sqlany_client_version( client_version, sizeof(client_version) );
	}
	if( !rc ) {
	    php_sprintf( client_version, "Unknown" );
	}
	php_info_print_table_row(2, "SQLAnywhere client version", client_version);
    } else {
	php_info_print_table_row(2, "SQLAnywhere client version", DBCAPI_NOT_FOUND_ERROR );
    }
    php_info_print_table_end();

}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
