/*-----------------------------------------------------------------------------+
 * Copyright 2008-2010 iAnywhere Solutions, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *                                                                                
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * 
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * While not a requirement of the license, if you do modify this file, we
 * would appreciate hearing about it. Please email sqlany_interfaces@sybase.com
 *-----------------------------------------------------------------------------+
 */

#ifndef PHP_PDO_SQLANYWHERE_INT_H
#define PHP_PDO_SQLANYWHERE_INT_H

#include "sacapidll.h"
#include "sacapi.h"

/* Represents error info */
typedef struct {
    char sqlstate[6];
    char errmsg[SACAPI_ERROR_SIZE];
    int errcode, line;
    const char* file;
} pdo_sqlanywhere_errinfo;

/* Holds representation of connection and error info */
typedef struct {
    a_sqlany_connection *dbc;
    pdo_sqlanywhere_errinfo einfo;
} pdo_sqlanywhere_db_handle;

/* Holds information for a single parameter */
typedef struct {
    a_sqlany_bind_param desired; // sqlany desired param vals
    a_sqlany_bind_param input;	// param to be bound as input
    a_sqlany_bind_param output;	// param to be bound as output
} pdo_sqlanywhere_param;

/* Holds information for a single column */
typedef struct {
    short* is_unsigned;
    unsigned int has_info;
    a_sqlany_column_info info;
} pdo_sqlanywhere_col;

/* Holds representation of a statement and error info*/
typedef struct {
    a_sqlany_stmt *stmt;
    unsigned long cur_row, num_params;
    pdo_sqlanywhere_errinfo einfo;
} pdo_sqlanywhere_stmt;

extern pdo_driver_t pdo_sqlanywhere_driver;
extern struct pdo_stmt_methods sqlanywhere_stmt_methods;

/* void because of build order. Avoids compiler complaining about contents */
void pdo_sqlanywhere_error (pdo_dbh_t *dbh, pdo_stmt_t *stmt, 
	const char *file, int line TSRMLS_DC);
#define DB_HANDLE_ERR(dbh) pdo_sqlanywhere_error(dbh, NULL, __FILE__, __LINE__ TSRMLS_CC)
#define DB_STMT_ERR(stmt) pdo_sqlanywhere_error(stmt->dbh, stmt, __FILE__, __LINE__ TSRMLS_CC)

#endif
